package scripts;

import jinro.*;
import scripts.roles.*;
import java.text.Collator;
import com.skype.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;

// Game class must inherit this class.

abstract class System {
	def strings = JinroLocalization.getBundle("System.properties");
	// List of Actors	参加者のリストです。
	def actors = [];
	// Dead's history	死者のリストです。
	def history = [];
	// First victim
	// This property is set by JinroGame.setFirstVictim(GroovyObject actor).	初日犠牲者を表します。
	def firstvictim;
	def roles = [];			//役職
	def cupidnum = 0;		//キューピットの人数
	def deathnotenext;			//次のデスノートを持つ予定の人
	def spirits = [];		//霊界チャットに入れる予定の人
	def spiritschatlist = [];	//霊界チャットに追加した人（２重追加防止用）
//	def revivallist = [];	//生き返った人リスト（霊界から追い出すリスト）
	volatile static def spiritschat = null;
	def wolveschat = null;
	// Name of times
	// This property is referred by JinroGame.addLog(String message) etc.
	//議論時間・夜時間それぞれの名称のリストです。
	def timename = [strings.getString("time.daytime"), strings.getString("time.nighttime")];
	def checkstepnum = -1;	//checkStepがまだ終わってない数の記憶
	def initialize = 0;		//初期化用 0=まだ　1=やった
	def check;				//投票や役職のチェック用（まだの人が入る）

	// Returns all state names.
	// This method is called by JinroGame.getStateNames().
	def getStateNames() {
		def list = [];
		State.each { list << it.name };
		return list;
	}
	// Returns the proper role class for the role's name.
	// This method is called by JinroGame.getRole(String name).
	def getRole(name) {
		return roles.find { it.newInstance().getRoleName() == name };
	}
	// Returns the proper state object for the state's name.
	// This method is called by JinroGame.getState(String name).
	def getState(name) {
		return State.find { it.name == name };
	}
	// Adds an actor.
	// This method is called by JinroGame.addActor(String name, String skypeid).
	//参加者を追加します。
	synchronized def addActor(name, skypeid = null) {
	//	noticeall("name="+name);
		if(actors.find { it.name == name }) {
			return false;
		}
		Actor newactor = new Actor(name);
		if(skypeid) {
			if(actors.find { it.skypeid == skypeid }) {
				return false;
			}
			newactor.skypeid = skypeid;
		}
		actors << newactor;
		def mes = strings.getString("entry.added", newactor.name, actors.size() );
		//entry.added=%Sさんを追加しました。現在%S名です。
		noticeall(mes);
		if(skypeid != null) { JinroSkype.notice(skypeid, mes); }
		return true;
	}
	// Removes an actor.
	// This method is called by JinroGame.removeActor(String name)
	def removeActor(name) {
		def actor = actors.find { it.name == name };
		actors.remove(actor);
		noticeall(strings.getString("entry.removed", actor.name, actors.size() ));
		//entry.removed=%Sさんを削除しました。現在%S名です。
	}
	// Returns the proper actor object for the actor's name.
	// This method is called by JinroGame.getActor(String name)
	def getActor(name) {
		return actors.find { it.name == name };
	}
	// Checks if the name already exists as an actor's.
	// This method is called by JinroGame.checkNameDuplication(String str).
	def checkNameDuplication(name) {
		return !actors.find { it.name == name };
	}
	// Returns the proper name for the Skype ID.
	// This method is called by JinroGame.getNamebySkypeid(String skypeid).
	def getNamebySkypeid(skypeid) {
		return actors.find { it.skypeid == skypeid }?.name;
	}
	// Deals the role to each of the actors.
	// This method is called by JinroGame.dealRoles().
	//役職を配布します。 
	def dealRoles() {
	//	println("aaaaaaaaa");
		noticeall(strings.getString("village.name"));
		//village.name=なかじま村
		Collections.shuffle(actors);	//参加者をシャッフル
		JinroGame.day = 0;
		JinroGame.time = 1;
		JinroGame.setProperty("firstdaytime",0);
		JinroGame.setProperty("gameend",false);
		JinroGame.onPhase = false;
		time0 = 0;
		spiritschatlist = [];
		spirits = [];
		spiritschat = null;
		cupidnum = 0;
		JinroGame.invokeMethod("resetActors");
		actors.each { 		//初期化
			it.killtime = -1;
			it.first = false;
			it.deathnote = false;			//デスノート初期化
			it.DTbyCivilian = false;		//村側のデビルトリガーを初期化
			it.cupid = 0;					//キューピットを初期化
			it.lovers = 0;					//恋人を初期化
			it.loverslive = true;			//道連れ判定用
			it.keep = 0;					//キープを初期化
			it.loversmark = "";
			it.state = State.live;
			it.statesave = it.state
			it.hoperolename = null;			//希望役職を初期化
			it.hoperolefinish = false;
			it.oldtargetname = null;
			it.mania = null;				//神話マニア初期化
		};
		def listroles = JinroGame.getProperty("listroles");
		listroles.clear();
		def deals = [];
		roles = JinroGame.getRoles();
		roles.findAll{ !it.is(Civilian) }.each{ role ->
			role.number.times { deals << role };					//設定した役職の人数を反映
			ListRoleName rolename = new ListRoleName();
			rolename.name2 = role.newInstance().getRoleName();		//配役名を取得
			rolename.name = JinroGame.getAlias( rolename.name2 );	//配役表示の為に役職名を取得
			rolename.order = role.order;											//ソート用に入れておいたが使わなかった
			rolename.number = role.number;											//配役表示の為の人数
			rolename.setnum = rolename.number;
			rolename.role = role;
			rolename.dealfinish = 0;								//配布済み人数
			listroles << rolename;
		}
		ListRoleName rolename = new ListRoleName();
		rolename.name2 = (new Civilian()).getRoleName();		//配役名を取得
		rolename.name = JinroGame.getAlias( rolename.name2 );	//配役表示の為に村人の役職名を取得
		rolename.order = Civilian.order;
		rolename.number = actors.size() - deals.size();							//配役表示の為に村人の人数
		rolename.setnum = rolename.number;
		rolename.role = Civilian;
		listroles << rolename;
		def liststr = strings.getString("rolelist") + " ";
		listroles.findAll{ it.number > 0 }.each {
			liststr += it.name + it.number + " ";
		}
		if(JinroGame.getProperty("roledisplay") == true) {
			noticeall(liststr);										//配役一覧を表示
	//		JinroGame.invokeMethod("optiondisplay");
		}
		(actors.size() - deals.size()).times { deals << Civilian };	//残りは村人
		
		if(JinroGame.getProperty("dealrolesrandom") == true) {		//役職は比率でランダム
		/*	def deals3 = [];
			def deals4 = [];
			def cv = [];
			def jingai = [];
			def i = 0;
			for(;;) {
				deals3.clear();
				deals4.clear();
				cv.clear();
				jingai.clear();
				deals4 = deals.findAll();
				cv = deals4.findAll{ it == Civilian };
				if(cv.size() >= 2) {	//最低限占いと狼はいるようにする
					deals4.remove(Civilian);
					deals3 << Fortuneteller;
					deals4.remove(Civilian);
					deals3 << Wolf;
				}
				deals4.each {
					deals3 << deals4.get(Math.floor(JinroGame.invokeMethod("rand") * deals4.size()) as int);
				}
				jingai = deals3.findAll{it==Wolf || it==Wolffortune || it==Wolfhead || it==Lunatic || it==Fanatic || 
					it==Fox || it==Foxnine || it==Minifox|| it==Apostate };	//人外
				if( ((Math.floor((deals.size()-1)/2.0+0.5) -1)  as int) >= jingai.size() ) break;	//PPチェッカー
				i++;
				if(i > 10 ) break;
			}//*/
			def deals3 = [];
			def jingai = [];
			def wolves = [];
			def fortunes = [];
			int i=0;
			for(i=0; i<100; i++) {	//フリーズ防止で100回（無茶な設定の場合はPPある）
				deals3.clear();
				jingai.clear();
				wolves.clear();
				fortunes.clear();
				deals.each {
					deals3 << deals.get(Math.floor(JinroGame.invokeMethod("rand") * deals.size()) as int);
				}
				jingai = deals3.findAll{it==Wolf || it==Wolffortune || it==Wolfhead || it==Lunatic || it==Fanatic || 
					it==Fox || it==Foxnine || it==Minifox|| it==Apostate };	//人外
				wolves = deals3.findAll{it==Wolf || it==Wolffortune || it==Wolfhead };	//狼
				fortunes = deals3.findAll{it==Fortuneteller || it==Mindreader };	//占い
				if( ((Math.floor((deals.size()-1)/2.0+0.5) -1)  as int) > jingai.size() ) {	//PPチェッカー
					if( (wolves.size() > 0) && (fortunes.size() > 0) ) break;				//占い＆狼１以上チェック
				}
			}
			JinroGame.addLog("試行回数" + i);
			deals = deals3;
			noticeall(strings.getString("dealt.random"));
			//dealt.random=配役は比率に応じてランダムになります。
			
			//比率の役職の人数を反映
			listroles.each { roles ->
				roles.number = 0;
				deals.each {
					if(roles.role == it) {
						roles.number++;
					}
				}
			}
		}
		def deals2 = deals.findAll { !it.bite };					//初日死ぬ役職をdeals2に
		def choice;
		def killkind;
		if(deals2) {
			choice = deals2.get(Math.floor(JinroGame.invokeMethod("rand") * deals2.size()) as int);	//初日犠牲の役職を決定
			killkind = State.bitten;
		} else {
			choice = deals.get(Math.floor(JinroGame.invokeMethod("rand") * deals.size()) as int);	//初日犠牲者がいない時は適当に決定
			killkind = State.suddendeath;	//特殊な場合は突然死
		}
		deals.remove(choice);										//初日犠牲者の役職を抜く
		firstvictim.setRole(choice);								//初日犠牲者に役職割り当て
		if( firstvictim.roleIs(new Fortuneteller()) ) {
			if(JinroGame.getProperty("firstfortune") == true) {			//初日犠牲者が占いの時、占いを行うか
				def civilian3 = actors.findAll{ !it.is(firstvictim) };
				civilian3.get(Math.floor(JinroGame.invokeMethod("rand") * civilian3.size()) as int).divined = true;
			}
		}
		firstvictim.killed(killkind);								//初日犠牲者を噛んで殺す
		firstvictim.killtime = 0;
		firstvictim.first = true;
		
		//オプション表示
		if(JinroGame.getProperty("roledisplay") == true) {
			JinroGame.invokeMethod("optiondisplay");
		}
		JinroGame.onPhase = false;
		history.clear();
		Collections.shuffle(actors);	//参加者を再度シャッフル
		// Notifies all actors of each of their names.  参加者名の表示
		def sortedactors = actors.clone();
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		//ソートしないようにする。
	//	sortedactors.sort { one, two -> collator.compare(one.name, two.name) };
		noticeall(strings.getString("notice.entries.start"));
		def clipboard = new StringBuilder(10 * sortedactors.size());
		clipboard << strings.getString("notice.entries.start") << "\n";
		//notice.entries.start=【参加者】
		def noticebuf0 = [];
		def noticebuf1 = [];
		def waittime = JinroGame.getProperty("waittime");
		sortedactors.each {
			noticebuf0 << (it.name);
			clipboard << it.name << "\n";
		}
		noticebuf1 << (strings.getString("notice.entries.end", sortedactors.size()));
		clipboard << strings.getString("notice.entries.end", sortedactors.size());
		//notice.entries.end=【計%S人】
		JinroInterface.copyClipboard(clipboard.toString().denormalize());
		//ラグ対策でウエイトを入れる為に別スレッド動作
		new Thread() {
			public void run() {
				Thread.sleep(waittime);
				noticebuf0.each {
					noticeall(it);
				}
				Thread.sleep(waittime);
				noticebuf1.each {
					noticeall(it);
				}
			}
		}.start();
		
		if( JinroGame.getProperty("hoperole") == false ) {
								//普通の役職配布
			actors.findAll{ !it.is(firstvictim) }.each {				//初日犠牲者以外の参加者を探す
				it.setRole(deals.remove(Math.floor(JinroGame.invokeMethod("rand") * deals.size()) as int));	//役職設定
				it.state = State.live;
				it.statesave = it.state;
				it.nextstate = State.live;
				it.countdown = 0;
			};
			dealRoles2();
		} else {				//希望役職
			JinroGame.setProperty("firstdaytime",3);
			
			actors.findAll{ !it.is(firstvictim) }.each {				//初日犠牲者以外の参加者を探す
				it.setRole(Civilian);	//全員村人に戻す
			}
		}
	}
	
	def dealRoles2() {		//主に役職を振った後の処理
		def civilian2 = actors.findAll { it.roleIs(new Civilian()) && it.isLive() };	//初日犠牲者以外の素村
		//キューピット＆求愛者を数えて番号を振る。
		actors.findAll { ( it.roleIs(new Cupid()) || it.roleIs(new Wooer()) || it.roleIs(new Wickedwoman()) ) && it.isLive() }.each {
			it.cupid = 1 << cupidnum;
			cupidnum++;
		}
		//ランダム恋人
		def randomlovers = JinroGame.getProperty("randomlovers");
	//	randomlovers = 2;
		def sellovers = actors.findAll { (it.lovers == 0) && it.isLive() };
		for(def i=0; i<randomlovers; i++) {
			def sel1 = sellovers.get(Math.floor(JinroGame.invokeMethod("rand") * sellovers.size()) as int);
			sellovers.remove(sel1);
			def sel2 = sellovers.get(Math.floor(JinroGame.invokeMethod("rand") * sellovers.size()) as int);
			sellovers.remove(sel2);
			cupidnum++;
			sel1.lovers = 1 << cupidnum;
			sel2.lovers = 1 << cupidnum;
			
			if(sel1.skypeid) {
				JinroSkype.notice(sel1.skypeid, strings.getString("randomlovers.partner", sel2.name ));
				//randomlovers.partner=愛の矢があなたに命中しました。あなたの恋人は%Sさんです。
			}
			if(sel2.skypeid) {
				JinroSkype.notice(sel2.skypeid, strings.getString("randomlovers.partner", sel1.name ));
				//randomlovers.partner=愛の矢があなたに命中しました。あなたの恋人は%Sさんです。
			}
			
			def skypeids = [];
			if(sel1.skypeid != null) { skypeids << sel1.skypeid; }
			if(sel2.skypeid != null) { skypeids << sel2.skypeid; }
			if(skypeids.size() >= 2) {
				JinroSkype.notice(skypeids as String[], 
					strings.getString("chat.created", "恋人" ));
				//chat.created=%Sチャットです。チャット捕捉のために何か言葉を打ち込んでください。
			}
			if(sellovers.size() < 2) {
				break;
			}
		}
		
		if( civilian2 && (JinroGame.getProperty("deviltrigger") == true) ) {
			//村側のデビルトリガーを一応選出
			civilian2.get(Math.floor(JinroGame.invokeMethod("rand") * civilian2.size()) as int).DTbyCivilian = true;
		}
		def lives = actors.findAll { it.isLive() };	//初日犠牲者以外
		if( lives && (JinroGame.getProperty("usedeathnote") == true) ) {
			//デスノートを選出
			deathnotenext = lives.get(Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int);
		}
		if( actors.findAll { it.roleIs(new Foxnine()) } ) {
			JinroGame.setProperty("existnine", true);		//九尾フラグ
		} else {
			JinroGame.setProperty("existnine", false);
		}
		
		JinroGame.day = 0;
		JinroGame.time = 1;
		if(JinroGame.getProperty("firstdaytime") != 3) {
			if(cupidnum == 0) {								//キューピット（求愛者）がいるか？
				JinroGame.setProperty("firstdaytime",0);	//いないなら普通
			} else {
				JinroGame.setProperty("firstdaytime",1);	//いるなら恋人決め時間
			}
		}
		def bakers = actors.findAll{ it.roleIs(new Baker()) };
		JinroGame.setProperty("oldbakersize",bakers.size());	//パン屋の人数を初期化
		
		noticeall(strings.getString("dealt"));
		//dealt=役職を配布しました。
		noticeall(strings.getString("dealt.res"));
		//dealt.res=役職が届いたら確認の為に返事をGMの個人チャットに送って下さい。
		
		actors.findAll { ( it.roleIs(new Absolute()) ) && it.isLive() }.each {
			noticeall(strings.getString("dealt.role", it.name, JinroGame.getAlias( (new Absolute()).getRoleName() ) ));
			//dealt.role=%Sさんは%Sです。
		}
	}
	
	
	
	// Add to the dead's history.
	// This method is called by JinroGame.addHistory(Object actor).
	//参加者を死者履歴に追加します。
	def addHistory(actor) {
		history << actor;
		if(actor.killtime < 0) {
			actor.killtime = JinroGame.getProperty("time");
		}
		//生死や役職がJinroSEから変更されたのを検出する為に保存
		actor.statesave = actor.state;
		actor.rolenamesave = actor.getRoleName();
		//霊界参加者をリストにいれる
		if(actor.skypeid) {
			spirits << actor;
		}
	/*	if(JinroGame.getProperty("autospiritschat") == true) {
			if(actor.skypeid) {
				def spiritsworld = history.findAll{ it.skypeid };
				def skypeids = [];
				spiritsworld.each { skypeids << it.skypeid }
				if(spiritsworld.size() == 2) {
					spiritschat = JinroSkype.notice(skypeids as String[], "霊界");
					spiritschat.setTopic("霊界");
				}
				if(spiritsworld.size() > 2) {
					User id = new User(spiritsworld[spiritsworld.size()-1].skypeid);
					spiritschat.addUser( id );
				}
			}
		}*/
	}
	//生き返った人用
	def delHistory(actor) {
		history.findAll{ it.name == actor.name }.each{ history.remove(it) };
		spirits.findAll{ it.name == actor.name }.each{ spirits.remove(it) };
		//生死や役職がJinroSEから変更されたのを検出する為に保存
		actor.statesave = actor.state;
		actor.rolenamesave = actor.getRoleName();
		spiritschatlist.findAll{ it.name == actor.name }.each{ spiritschatlist.remove(it) };
		if(spiritschat != null) {
			spiritschat.send(String.format("/kick %s", actor.skypeid) );
		}
		//２人以下は霊界破棄して作り直し
		if(spiritschatlist.size() < 2) {
			spiritschatlist.each{
				spirits << it;
				spiritschat.send(String.format("/kick %s", it.skypeid) );
			}
			spiritschat = null;
			spiritschatlist = [];
		}
	}
	// Notifies the each of actors of his/her role.
	// This method is called by JinroInterface.makeDialogStartGame().
	//各々の参加者に対しそれぞれの役職を通知します。またmakeChat()を呼び出します。 
	//このメソッドはツール本体から呼び出されるため必須です。
	def noticeRoles() {
		actors.each { it.noticeRole() };
		makeChats();
	}
	// Makes group chats.
	def makeChats() {
		// For Werewolves
		def wolves = actors.findAll { it.isWolf() && it.skypeid };
		if(wolves.size() > 1) {
			def skypeids = [];
			wolves.each { skypeids << it.skypeid };
			wolveschat = JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Wolf()).getRoleName())));
			wolveschat.setTopic(strings.getString("chat.topic", JinroGame.getAlias((new Wolf()).getRoleName())));
		}
		// For Sharers
		def sharers = actors.findAll { it.roleIs(new Sharer()) && it.skypeid };
		if(sharers.size() > 1) {
			def skypeids = [];
			sharers.each { skypeids << it.skypeid };
			JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Sharer()).getRoleName()))).setTopic(strings.getString("chat.topic", JinroGame.getAlias((new Sharer()).getRoleName())));
		}
		//妖狐チャット
		def foxchat = JinroGame.getProperty("foxchat");
		def foxes = actors.findAll { ( it.roleIs(new Fox()) || it.roleIs(new Foxnine()) ) && it.skypeid };
		if(foxes.size() > 1 && foxchat) {
			def skypeids = [];
			foxes.each { skypeids << it.skypeid };
			JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Fox()).getRoleName()))).setTopic(strings.getString("chat.topic", JinroGame.getAlias((new Fox()).getRoleName())));
		}
		//子狐チャット
		def minifoxchat = JinroGame.getProperty("minifoxchat");
		def minifoxes = actors.findAll { ( it.roleIs(new Minifox()) ) && it.skypeid };
		if(minifoxes.size() > 1 && minifoxchat) {
			def skypeids = [];
			minifoxes.each { skypeids << it.skypeid };
			JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Minifox()).getRoleName()))).setTopic(strings.getString("chat.topic", JinroGame.getAlias((new Minifox()).getRoleName())));
		}
	}
	// Sends the command for voting or any actions.
	// This method is called by JinroGame.sendMessage(String name, String type, String target).
	//参加者のコマンドを処理します。name - 参加者名  type - コマンド名  target - コマンドの対象
	synchronized def sendMessage(name, type, target) {
	//	noticeall("name="+name+" type="+type+" target="+target);
	//	JinroGame.addLog("type="+type+" target="+target);
		def actor = actors.find { it.name == name };
		def targetactors = target ? actors.findAll { it.name.contains(target) } : [];
		targetactors.sort { one, two -> one.name.size() <=> two.name.size() };
		if(actor?.isLive()) {
			//希望役職
			if(JinroGame.time + JinroGame.getProperty("firstdaytime")*2 == 7) {
				actor.receiveMessage(type, target);
				actor.suspended = null;
				return;
			}
			def targetname = target;
			def target0 = null;
			if(targetactors) {
				target0 = targetactors[0];
			}
			switch(JinroGame.time) {
				case 1:	//夜
					if(type == strings.getString("command.leavingseat")) {
					//command.leavingseat=離席
						if(JinroGame.getProperty("leavingseat") == true) {
							if(actor.skypeid != null) {
								JinroSkype.notice(actor.skypeid, "→"+strings.getString("command.leavingseat") );
							}
							JinroGame.addLog(strings.getString("leavingseat.log", name ));
							//leavingseat.log=%Sさんが離席しました。ＧＭは参加者のチャットを確認して下さい。
							final Shell shell = JinroInterface.getMainwindow();
							final String str = strings.getString("leavingseat.dialog");
							//leavingseat.dialog=離席者がいます。ＧＭは参加者のチャットを確認して下さい。
							Display.getDefault().asyncExec(new Runnable() {
								public void run() {
									MessageBox msg = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
									msg.setMessage(strings.getString(str));
									msg.open();
								}
							});
							
						}
					}
			}
			if(type == strings.getString("command.toasted")) {
				//command.toasted=焼き
				target0 = null;
			}
			if(targetname && target0 && target0.name != targetname) {	//名前が違う場合？
				actor.notice(strings.getString("confirm.commandtarget", type, target0.name));
				//confirm.commandtarget=%1$Sの対象を%2$Sさんと見なします。正しい場合は「%1$S＠」とだけ記入してください。間違っている場合は正しくなるように記入し直してください。
				actor.suspended = [ type: type, target: target0 ];
			} else if(!targetname && actor.suspended?.type == type) {
				actor.receiveMessage(type, actor.suspended.target);
				actor.suspended = null;
			} else if(target0 != null) {
				actor.receiveMessage(type, target0);
				actor.suspended = null;
			} else {
				actor.receiveMessage(type, target);
				actor.suspended = null;
			}
			check = actors.findAll { (it.checkStep() == false) && it.isLive() };
			def mes;
			switch(JinroGame.time) {
				case 0:
					mes = strings.getString("com0.t0")
					//com0.t0=投票がまだの人が
					break;
				case 1:
					mes = strings.getString("com0.t1")
					//com0.t1=役職実行がまだの人が
					break;
			}
		//	def checkstepnum = JinroGame.getProperty("checkstepnum");
			if(checkstepnum != check) {
				switch(check.size()) {
					case 0:
						switch(JinroGame.time) {
							case 0:
								JinroGame.addLog( strings.getString("com1.t0") );
								//com1.t0=投票が全て終わりました(｀・ω・´)
								if( (JinroGame.getProperty("votecompletiondialog") == true) ) {
									final Shell shell = JinroInterface.getMainwindow();
									final String str = strings.getString("com1.t0");
									Display.getDefault().asyncExec(new Runnable() {
										public void run() {
											MessageBox msg = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
											msg.setMessage(strings.getString(str));
											msg.open();
										}
									});
								}
								break;
							case 1:
								JinroGame.addLog( strings.getString("com1.t1") );
								//com1.t1=役職実行が全て終わりました(｀・ω・´)
								if( (JinroGame.getProperty("rolecompletiondialog") == true) && (JinroGame.day != 0) ) {
									final Shell shell = JinroInterface.getMainwindow();
									final String str = strings.getString("com1.t1");
									Display.getDefault().asyncExec(new Runnable() {
										public void run() {
											MessageBox msg = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
											msg.setMessage(strings.getString(str));
											msg.open();
										}
									});
								}
								break;
						}
						break;
						//com2.0=人います。
						//com2.1=さんと
						//com2.2=さんがまだです
					case 1:
						JinroGame.addLog(mes+check.size()+strings.getString("com2.0")+check[0].name+strings.getString("com2.2") );
						break;
					case 2:
						JinroGame.addLog(mes+check.size()+strings.getString("com2.0")+check[0].name+strings.getString("com2.1")
																							+check[1].name+strings.getString("com2.2") );
						break;
					case 3:
						JinroGame.addLog(mes+check.size()+strings.getString("com2.0")+check[0].name+strings.getString("com2.1")
													+check[1].name+strings.getString("com2.1")+check[2].name+strings.getString("com2.2") );
						break;
				}
				checkstepnum = check;
			}
		}
	}
	// Notifies all actors of the message.
	def noticeall(message) {
		JinroGame.addLog(strings.getString("notice.log", message));
		JinroSkype.noticeAll(message);
	}
}

class ListRoleName {
	ListRoleName() {};
	def name = "";			//役職名
	def name2 = "";			//真の役職名(alias前)
	def order = 0;
	def number = 0;			//役職人数
	def setnum = 0;			//設定されてる人数
	def role;				//役職のクラス
	def dealfinish = 0;		//配布済み人数
}
