package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Apostate
// Camp: Foxes

class Apostate extends Civilian {
	def static order = 17;
	def static strings = Civilian.strings.attachBundle("roles/Apostate.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	Apostate(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 58;
	}
	// Update priority
	// Returns under 0: target -> it
	// Returns over 0 : it -> target
	def getUpdatePriority(target) {
		if(target.roleIs(new Fox()) || target.roleIs(new Minifox()) || target.roleIs(new Foxnine())) {
			return -1;
		} else {
			return 0;
		}
	}
	def getRoleName() {
		return strings.getString("apostate");
		//apostate=背徳者
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase()
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) { break; }
				if(!JinroGame.getProperty("existnine")) { break; }
				notice(strings.getString("notice.pray"));
				//notice.pray=念殺先を「念殺＠対象の名前」の形式で夜時間の間に決定してください。
				notice(strings.getString("notice.pray2"));
				//notice.pray2=但し、念殺しない事を選択する事も出来ます。
				def actors = JinroGame.getActors();
				def first = actors.findAll { (it.first == true) };
				def GM = "初日犠牲者"
				if(first) {
					GM = first[0].name;
				}
				notice(strings.getString("notice.pray3",GM));
				//notice.pray3=念殺をしない場合は「念殺しない＠%S」として下さい。
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(!JinroGame.getProperty("existnine")) { break; }
				if(JinroGame.day == 0) { break; }
				if(target) { break; }
				if(this.sealtarget == true)  { break; }
				actors.findAll{ it.name != name && it.isLive() }.each {
					commands << strings.getString("command.pray") + "@" + it.name;
					//command.pray=念殺
				}
				actors.findAll{ (it.name != name) && (it.first == true) }.each {
					commands << strings.getString("command.pray2") + "@" + it.name;
					//command.pray2=念殺しない
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target)
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(!JinroGame.getProperty("existnine")) { return; }
				if(JinroGame.day == 0) { return; }
				if(this.target) { return; }
				if(this.sealtarget == true)  { return; }
				if(type == strings.getString("command.pray")) {
					//command.pray=念殺
					if(target?.name != name && target?.isLive()) {
						this.target = target;
						notice(strings.getString("notice.pray.target", target.name));
						//notice.pray.target=念殺先→%Sさん
					}
				}
				if(type == strings.getString("command.pray2")) {
					//command.pray2=念殺しない
					this.sealtarget = true;
					notice(strings.getString("notice.pray2.target", target.name));
					//notice.pray2.target=念殺しない→%Sさん
				}
				break;
		}
	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if(!JinroGame.getProperty("existnine")) { return true; }
				if(JinroGame.day == 0) { return true; }
				if(target) { return true; }
				if(this.sealtarget == true)  { return true; }
				break;
		}
		return false;
	}
	// Notifies the actor of the role.
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.roleIs(new Fox()) || it.roleIs(new Foxnine()) }.each {
			notice(strings.getString("notice.fox", it.name, JinroGame.getAlias(it.getRoleName())));
			//notice.fox=%1$Sさんが%2$Sです。
		}
	}
	// Goes to next phase.
	// If there aren't any foxes, the actor dies.
	def update() {
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				//念殺
				def actors = JinroGame.getActors();
		//		actors.findAll{ it.isLive() && it.roleIs(new Foxnine()) }.each {
				actors.findAll{ it.roleIs(new Foxnine()) }.each {
					if( (it.target?.name == this.target?.name) && (this.target != null) ) {
						killed(State.sacrifice);
						this.target.killed(State.pray);
					}
				}
				//妖狐陣営が全滅した場合、自分も死ぬ
				def foxes = JinroGame.getActors().findAll { (it.roleIs(new Fox()) || it.roleIs(new Minifox()) || it.roleIs(new Foxnine())) && it.isLive() };
				if(!foxes) {
					nextstate = State.followed;
					countdown = 1;
				}
		}
		super.update();
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	//勝利結果 0=村 1=狼 2=狐 3=恋人 4=狐＋恋人 return true=勝利 false=敗北
	def judge(result) {
		if(lovers != 0) {
			def lover = JinroGame.getActors().findAll { ((it.lovers & this.lovers) != 0) && it.isLive() };
			def lovercount = [];
			def lovermax = 0;
			lover.each {
				if(lovercount[it.lovers] == null) lovercount[it.lovers] = 1;
				lovercount[it.lovers]++;
				if(lovercount[it.lovers] > lovermax) {
					lovermax = lovercount[it.lovers];
				}
			}
			if(lovermax >= 2) {
				return (result == 3) || (result == 4);	//恋人
			} else {
				return false;
			}
		}
		return (result == 2) || (result == 4);		//狐の場合
	}
}
