package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Cat
// Camp: Civilians

class Cat extends Civilian {
	def static order = 10;
	def static strings = Civilian.strings.attachBundle("roles/Cat.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = true;		// All roles must define own "bite" property.
	def static isInvisible() {
		return true;
	}
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	Cat(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 30;
	}
	// Update priority
	// Returns under 0: target -> it
	// Returns over 0 : it -> target
	def getUpdatePriority(target) {
		if(target.isWolf()) {
			return 1;
		} else {
			return 0;
		}
	}
	def getRoleName() {
		if(JinroGame.getProperty("catascetic") == true) {
			return strings.getString("ascetic");
			//cat=修験者
		} else {
			return strings.getString("cat");
			//cat=猫又
		}
	}

	def noticeRole() {
		if(JinroGame.getProperty("catascetic") == true) {
			notice(strings.getString("notice.role", strings.getString("civilian")));
		} else {
			notice(strings.getString("notice.role", strings.getString("cat")));
			//cat=猫又
		}
	}

	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				if(JinroGame.getProperty("catascetic") == true && JinroGame.day == 2) {
					notice(strings.getString("notice.role.ascetic2"));
					notice(strings.getString("notice.role.ascetic", strings.getString("cat")));
					//notice.role.ascetic=あなたは%Sでした。
					notice(strings.getString("notice.role.ascetic2"));
				}
				if(JinroGame.getProperty("catrevival") == true) {
					notice(strings.getString("notice.revival"));
					//notice.revival=蘇生先を「蘇生＠対象の名前」の形式で夜時間の間に決定してください。
					notice(strings.getString("notice.revival2"));
					//notice.revival2=蘇生しない場合は「蘇生しない＠」として下さい。
				}
				break;
		}
	}
	// Returns the actor's command list
	//ツール本体から手動でコマンドを選択するために、そのリストを返します。
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(statesave != State.live) return commands;
				if(!target) {
					if(JinroGame.getProperty("catrevival") == true) {
						def firstvictim = JinroGame.getProperty("firstvictim");		//初日犠牲者
						actors.findAll{ it.name != name && (!it.isLive()) && it.name != firstvictim.name }.each {
							commands << strings.getString("command.revival") + "@" + it.name;
							//command.revival=蘇生
						};
						commands << strings.getString("command.norevival") + "@";
						//command.norevival=蘇生しない
					}
				}
				break;
		}
		return commands;
	}
	//コマンドを処理します。昼時間の投票の他、夜時間の護衛を処理します。
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				if(JinroGame.getProperty("catrevival") == true) {
					if(this.target) {
						return;
					}
					if(type == strings.getString("command.revival")) {
						//command.revival=蘇生
						def firstvictim = JinroGame.getProperty("firstvictim");		//初日犠牲者
						if(target?.name != name && (!target?.isLive()) && target?.name != firstvictim.name) {
							this.target = target;
							notice(strings.getString("notice.revival.target", target.name));
							//notice.revival.target=蘇生先→%Sさん
							
							//蘇生対象を保存（蘇生自体は夜明けだけど、ここで抽選）
							if(target) {
								def actors = JinroGame.getActors();
								def r0 = Math.floor(JinroGame.invokeMethod("rand") * 100) as int;
								def r1 = Math.floor(JinroGame.invokeMethod("rand") * 100) as int;
								def choice = null;
								if((0 <= r0) && (r0 < 66)) {
									//通常の蘇生
									choice = target;
								} else {
									//誤爆蘇生
									def gobaku = actors.findAll{(it.name != name) && (!it.isLive()) && (it.name != firstvictim.name)};
									if(gobaku.size() > 0) {
										choice = gobaku[Math.floor(JinroGame.invokeMethod("rand") * gobaku.size()) as int];
									}
								}
								if(choice != null) {
									revivaltarget = choice;
								}
								revivalsuccess = false;
								if((0 <= r1) && (r1 < 33)) {
									//蘇生成功
									revivalsuccess = true;
								}
							}
						}
					}
					if(type == strings.getString("command.norevival")) {
						//command.norevival=蘇生しない
						this.norevival = true;
						notice(strings.getString("notice.norevival"));
						//notice.norevival=蘇生しない→
					}
				}
				break;
		}
	}
	def update() {
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				if(JinroGame.getProperty("catrevival") == true) {
					if(revivaltarget) {
						if(revivalsuccess == true) {
							//蘇生処理
							revivaltarget.state = State.live;
							noticeKilled(strings.getString("notice.revived", revivaltarget.name) );
							//notice.revived=%Sさんが奇跡的に生き返りました。
							JinroGame.invokeMethod("delHistory",revivaltarget);
						}
						if(norevival != true) {
							def mess;
							if(revivalsuccess == true) {
								mess = strings.getString("notice.spiritschat.success", revivaltarget.name);
								//notice.spiritschat.success=%Sさんの蘇生に成功しました。
							} else {
								mess = strings.getString("notice.spiritschat.failure", revivaltarget.name);
								//notice.spiritschat.failure=%Sさんの蘇生に失敗しました。
							}
							//霊界に表示
							JinroGame.invokeMethod("spiritschatsend",mess);
						}
					}
				}
				break;
		}
		super.update();
	}
	// Checks if the game can go to next phase.
	//ゲームを進められるかどうかを返します。必要な能力の使用が済んだかどうかなどで判断します。
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if(JinroGame.getProperty("catrevival") == false) {
					return true;
				}
				if(JinroGame.day == 0) {
					return true;
				}
				if(target) {
					return true;
				}
				if(this.norevival == true) {
					return true;
				}
				if(statesave != State.live) return true;
				break;
		}
		return false;
	}

	// When the actor is killed.
	def killed(state) {
		def actors = JinroGame.getActors();
		def loversfellow = JinroGame.getProperty("loversfellow");	//恋人道連れ
		def cathang = JinroGame.getProperty("cathang");				//猫吊り即るー
		def catbitecurse = JinroGame.getProperty("catbitecurse");	//猫噛み狼死亡
		if(state != State.executed) {
			if(state == State.bitten) {		//噛みの場合
				def wo = actors.findAll { it.isWolf() };
				if(lovers == 0) {		//恋人以外　狼を道連れ
					if(catbitecurse == true) {
						wo.each {	//噛みを実行した狼を道連れ
							if( (it.state == State.live) && (it.bitetarget2?.name == this.name) ) {
								it.killed(State.followed);		//道連れ
							}
						}
					} else {
						def lives = actors.findAll { it.isLive() && it.name != name && it.isWolf() };
						def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
						choice.nextstate = State.followed;
						choice.countdown = 1;
					}
				} else {					//恋人の時
					if(loversfellow == true) {	//恋人道連れ有効
						wo.each {	//噛みを実行した狼を道連れ
							if( (it.state == State.live) && (it.bitetarget2?.name == this.name) ) {
								it.killed(State.followed);		//道連れ
							}
						}
					} else {					//恋人道連れ無効の場合は猫として道連れ
						if(catbitecurse == true) {	//噛みを実行した狼を道連れ
							wo.each {
								if( (it.state == State.live) && (it.bitetarget2?.name == this.name) ) {
									it.killed(State.followed);		//道連れ
								}
							}
						} else {				//狼の中からランダムで道連れ（標準の奴）
							def lives = actors.findAll { it.isLive() && it.name != name && it.isWolf() };
							def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
							choice.nextstate = State.followed;
							choice.countdown = 1;
						}
					}
					//恋人の後追い自殺
					def lover = actors.findAll { ((it.lovers & this.lovers) != 0) && (it.name != name) };
					this.loverslive = false;						//無限ループ対策
					lover.each {
						if((it.state == State.live) && (it.loverslive)) {
							it.killed(State.followup);		//後追い
						}
					}
				}
			} else {
				if(lovers != 0) {		//恋人で吊り・噛み以外（射殺・ラブガード・道連れ等）
					//恋人の後追い自殺
					def lover = actors.findAll { ((it.lovers & this.lovers) != 0) && (it.name != name) };
				//	this.state = state;						//無限ループ対策
					this.loverslive = false;						//無限ループ対策
					lover.each {
						if((it.state == State.live) && (it.loverslive)) {
							it.killed(State.followup);		//後追い
						}
					}
				/*	if(lover && (state != State.followup)) {
						lover[0].killed(State.followup);		//後追い
					}*/
				}
			}
			if( (state != State.shooting) && (state != State.loveguard) && (state != State.deathnote) ) {
				if(this.state == State.live) {
					if( (deathnoteflg == false) || (state == State.followed) ) {//デスノで死んだら無残なを出さないが、道連れなら出す
						if(state == State.followup) {	//恋人後追いの時
							noticeKilled(strings.getString("notice.killed.lovers.followup", name) );
							//notice.killed.lovers.followup=%Sさんは恋人の後を追い自殺しました
						} else {
							noticeKilled(strings.getString("notice.killed.normal", name) );
							//notice.killed.normal=%Sさんが無残な死体となって発見されました・・・。
						}
					}
				}
			}
		} else {							//吊りの場合
			if(lovers == 0) {		//恋人以外
				def lives = actors.findAll { it.isLive() && it.name != name };
				def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
				
			//	////猫又の道連れが恋人の場合のデバッグテスト用
			//	def debug = lives.findAll{ it.lovers != 0 };//デバッグテスト用
			//	if(debug.size() == 2) { choice = debug[0]; }//デバッグテスト用
			//	////
				choice.nextstate = State.followed;
				if(cathang == true) {	//猫吊り即るー有効
					choice.killed(State.followed);
		//			choice.countdown = 1;
				} else {				//猫吊り即るー無効
					choice.countdown = 2;
				}
			} else {					//恋人の時
				if(loversfellow == true) {		//恋人の道連れがある場合
					def lives = actors.findAll { it.isLive() && (it.name != name) && (it.voteforlovers == true) };
					if(lives)  {			//投票した人を道連れ
						def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
						choice.nextstate = State.followed;
						choice.countdown = 2;
					}
				} else {						//恋人の道連れが無い場合は猫として道連れ
					def lives = actors.findAll { it.isLive() && it.name != name };
					def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
					choice.nextstate = State.followed;
					if(cathang == true) {	//猫吊り即るー有効
						choice.killed(State.followed);
					//	choice.nextstate = State.followed;
					//	choice.countdown = 1;
					} else {				//猫吊り即るー無効
						choice.nextstate = State.followed;
						choice.countdown = 2;
					}
				}
				//恋人の後追い自殺
				def lover = actors.findAll { ((it.lovers & this.lovers) != 0) && (it.name != name) };
				lover.each {
					if(it.state == State.live) {
						it.nextstate = State.followup;
						it.countdown = 2;
					}
				}
			/*	if(lover) {
					lover[0].nextstate = State.followup;
					lover[0].countdown = 2;
				}*/
			}
		}
		this.killtime = JinroGame.getProperty("time0");
		this.state = state;
		JinroGame.addHistory(this);
	}
	
	//自己蘇生
	def selfrevival() {
	/*	switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				def r = Math.floor(JinroGame.invokeMethod("rand") * 100) as int;
			//	notice("aaaaaaaaaaaaaaaaa1 r="+r);
				if((0 <= r) && (r < 20)) {
					this.state = State.live;
					noticeKilled(strings.getString("notice.selfrevived", this.name) );
					//notice.selfrevived=%Sさんが自己蘇生の結果生き返りました。
					JinroGame.invokeMethod("delHistory",this);
				}
				break;
		}//*/
	}
}
