package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Fortuneteller
// Camp: Civilians

class Fortuneteller extends Civilian {
	def static order = 2;
	def static strings = Civilian.strings.attachBundle("roles/Fortuneteller.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 1;
	}
	def getPriority() {
		return 50;
	}
	Fortuneteller(actor = null) {
		super(actor);
	}
	def getRoleName() {
		return strings.getString("fortuneteller");
		//fortuneteller=占い師
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					notice(strings.getString("notice.nodivine"));
					//notice.nodivine=初日は占えません。二日目より占う事が出来ます。
					break;
				}
				if( (JinroGame.getProperty("zerodayrandom") == true) && (JinroGame.day == 0) ) {
					//初日のランダム占い有り
					def actors = JinroGame.getActors().findAll { it.isLive() && (it.name != name) };
					if(JinroGame.getProperty("zerodayrandombeginners") == true) {	//初心者モードは村限定
						actors = actors.findAll { it.isCivilian() };
					}
					this.target = actors.get(Math.floor(JinroGame.invokeMethod("rand") * actors.size()) as int);
					notice(strings.getString("notice.divine.target", this.target.name));
					//notice.divine.target=占い先→%Sさん
					divine();
				} else {
					//初日のランダム占い無し
					notice(strings.getString("notice.divine"));
					//notice.divine=占い先を「占い＠対象の名前」の形式で夜時間の間に決定してください。
					if( (JinroGame.getProperty("zerodayrandomfortune") == true) && (JinroGame.day == 0) ) {
						notice(strings.getString("notice.divine2"));
						//notice.divine2=初日のみ占い先をランダムで選ぶ事が出来ます。
						notice(strings.getString("notice.divine3"));
						//notice.divine3=占い先をランダムで選ぶ場合は「占いランダム＠」としてください。
					}
				}
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					return commands;
				}
				if(statesave != State.live) return commands;
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.divine") + "@" + it.name;
						//command.divine=占い
					}
				}
				break;
		}
		return commands;
	}
	//コマンドを処理します。昼時間の投票の他、夜時間の占いを処理します。
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					if(JinroGame.getProperty("zerodayrandom") == true) {return;}
					if(JinroGame.getProperty("zerodaynofortune") == true) {return;}
				}
				if(this.target) { return; }
				if(type == strings.getString("command.divine")) {
				//command.divine=占い
					if(target?.name != name && target?.isLive()) {
						this.target = target;
						notice(strings.getString("notice.divine.target", target.name));
						//notice.divine.target=占い先→%Sさん
						divine();
					}
				}
				
				if( (JinroGame.getProperty("zerodayrandomfortune") == true) && (JinroGame.day == 0) ) {
					if(type == strings.getString("command.randomdivine")) {
					//command.randomdivine=占いランダム
						def actors = JinroGame.getActors().findAll { it.isLive() && (it.name != name) };
						if(JinroGame.getProperty("zerodayrandombeginners") == true) {	//初心者モードは村限定
							actors = actors.findAll { it.isCivilian() };
						}
						this.target = actors.get(Math.floor(JinroGame.invokeMethod("rand") * actors.size()) as int);
						notice(strings.getString("notice.divine.target", this.target.name));
						//notice.divine.target=占い先→%Sさん
						divine();
					}
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	//ゲームを進められるかどうかを返します。必要な能力の使用が済んだかどうかなどで判断します。
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					return true;
				}
				if(target) return true;
				if(statesave != State.live) return true;
				break;
		}
		return false;
	}
	//占いを実行します。相手が狼である場合は黒（●）、そうでなければ白（○）と判定されます。
	def divine() {
		target.divined = true;
		if(target.isWolf() && !target.roleIs(new Wolfhead())) {
				notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.black")));
		} else {
			notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.white")));
		}
	}
}
