package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Hunter
// Camp: Civilians

class Hunter extends Civilian {
	def static order = 5;
	def static strings = Civilian.strings.attachBundle("roles/Hunter.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		if(number > 10) {
			return 1;
		} else {
			return 0;
		}
	}
	def getPriority() {
		return 40;
	}
	Hunter(actor = null) {
		super(actor);
	}
	def getRoleName() {
		return strings.getString("hunter");
		//hunter=狩人
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				if(JinroGame.getProperty("norenzokuguard") == true) {
					notice(strings.getString("notice.defend2"));
				} else {
					notice(strings.getString("notice.defend"));
					//notice.defend=護衛先を「護衛＠対象の名前」の形式で夜時間の間に決定してください。
				}
				break;
		}
	}
	// Returns the actor's command list
	//ツール本体から手動でコマンドを選択するために、そのリストを返します。
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(statesave != State.live) return commands;
				if(!target) {
					if(JinroGame.getProperty("norenzokuguard") == true) {
						actors.findAll{ (oldtargetname != it.name) && (it.name != name) && it.isLive() }.each {
							commands << strings.getString("command.defend") + "@" + it.name;
							//command.defend=護衛
						};
					} else {
						actors.findAll{ it.name != name && it.isLive() }.each {
							commands << strings.getString("command.defend") + "@" + it.name;
							//command.defend=護衛
						};
					}
				}
				break;
		}
		return commands;
	}
	//コマンドを処理します。昼時間の投票の他、夜時間の護衛を処理します。
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				if(type != strings.getString("command.defend")) {
					//command.defend=護衛
					return;
				}
				if(this.target) {
					return;
				}
				if(JinroGame.getProperty("norenzokuguard") == true) {
					if(oldtargetname == target?.name) {
						notice(strings.getString("notice.defend.sametarget"));
						//notice.defend.sametarget=同じ人を連続で護衛出来ません。
						return;
					}
				}
				if(target?.name != name && target?.isLive()) {
					oldtargetname = target?.name;
					this.target = target;
					notice(strings.getString("notice.defend.target", target.name));
					//notice.defend.target=護衛先→%Sさん
					defend();
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	//ゲームを進められるかどうかを返します。必要な能力の使用が済んだかどうかなどで判断します。
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day == 0) {
					return true;
				}
				if(target) {
					return true;
				}
				if(statesave != State.live) return true;
				break;
		}
		return false;
	}
	//護衛を実行します。
	def defend() {
		target.defended = true;
	}
}
