package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Mini fox
// Camp: Foxes

class Minifox extends Civilian {
	def static order = 18;
	def static strings = Civilian.strings.attachBundle("roles/Minifox.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	def static isInvisible() {
		return false;
	}
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	Minifox(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 60;
	}
	// If it returns true, the role count as a civilian.
	def isCivilian() {
		return false;
	}
	def getRoleName() {
		return strings.getString("minifox");
		//minifox=子狐
	}
	//参加者に対し役職の通知を行います。
	// Notifies the actor of the role.
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ ( it.roleIs(new Fox()) || it.roleIs(new Foxnine()) ) }.each {
			notice(strings.getString("notice.fox", it.name, JinroGame.getAlias(it.getRoleName())));
			//notice.fox=%1$Sさんが%2$Sです。
		};
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase()
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					notice(strings.getString("notice.nodivine"));
					//notice.nodivine=初日は占えません。二日目より占う事が出来ます。
					break;
				}
				if( (JinroGame.getProperty("zerodayrandom") == true) && (JinroGame.day == 0) ) {
					//初日のランダム占い有り
					def actors = JinroGame.getActors().findAll { it.isLive() && (it.name != name) };
					if(JinroGame.getProperty("zerodayrandombeginners") == true) {	//初心者モードは村限定
						actors = actors.findAll { it.isCivilian() };
					}
					this.target = actors.get(Math.floor(JinroGame.invokeMethod("rand") * actors.size()) as int);
					notice(strings.getString("notice.divine.target", this.target.name));
					//notice.divine.target=占い先→%Sさん
					divine();
				} else {
					//初日のランダム占い無し
					notice(strings.getString("notice.divine"));
					//notice.divine=占い先を「占い＠対象の名前」の形式で夜時間の間に決定してください。
					if(JinroGame.day == 0) {
						notice(strings.getString("notice.divine2"));
						//notice.divine2=初日のみ占い先をランダムで選ぶ事が出来ます。
						notice(strings.getString("notice.divine3"));
						//notice.divine3=占い先をランダムで選ぶ場合は「占いランダム＠」としてください。
					}
				}
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					return commands;
				}
				if(statesave != State.live) return commands;
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.divine") + "@" + it.name;
						//command.divine=占い
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target)
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					if(JinroGame.getProperty("zerodayrandom") == true) {return;}
					if(JinroGame.getProperty("zerodaynofortune") == true) {return;}
				}
				if(this.target) { return; }
				if(type == strings.getString("command.divine")) {
				//command.divine=占い
					if(target?.name != name && target?.isLive()) {
						this.target = target;
						notice(strings.getString("notice.divine.target", target.name));
						//notice.divine.target=占い先→%Sさん
						divine();
					}
				}
				if(type == strings.getString("command.randomdivine")) {
				//command.randomdivine=占いランダム
					if(JinroGame.day == 0) {
						def actors = JinroGame.getActors().findAll { it.isLive() && (it.name != name) };
						if(JinroGame.getProperty("zerodayrandombeginners") == true) {	//初心者モードは村限定
							actors = actors.findAll { it.isCivilian() };
						}
						this.target = actors.get(Math.floor(JinroGame.invokeMethod("rand") * actors.size()) as int);
						notice(strings.getString("notice.divine.target", this.target.name));
						//notice.divine.target=占い先→%Sさん
						divine();
					}
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if( (JinroGame.getProperty("zerodaynofortune") == true) && (JinroGame.day == 0) ) {
					return true;
				}
				if(target) {
					return true;
				}
				if(statesave != State.live) return true;
				break;
		}
		return false;
	}
	def divine() {
		//notice.divine.result.black=子狐の%Sくんは%Sさんを一生懸命占いました。やったー！●黒見つけたよ！！
		//notice.divine.result.white=子狐の%Sくんは%Sさんを一生懸命占いました。ほっ、○白だったよ
		//notice.divine.failed=子狐の%Sくんは%Sさんを一生懸命占いました。うんしょっ、うんしょっ・・・今日はダメみたい
		def minifoxprobability = JinroGame.getProperty("minifoxprobability");	//子狐の占い確率［％］
		def mini = JinroGame.getAlias(new Minifox().getRoleName());
		if(JinroGame.invokeMethod("rand")*100 < minifoxprobability) {
			if(target.isWolf() && !target.roleIs(new Wolfhead())) {
				notice(strings.getString("notice.divine.result.black", mini, this.name, target.name));
			} else {
				notice(strings.getString("notice.divine.result.white", mini, this.name, target.name));
			}
		} else {
			notice(strings.getString("notice.divine.failed", mini, this.name, target.name));
		}
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	//勝利結果 0=村 1=狼 2=狐 3=恋人 4=狐＋恋人 return true=勝利 false=敗北
	def judge(result) {
		if(lovers != 0) {
			def lover = JinroGame.getActors().findAll { ((it.lovers & this.lovers) != 0) && it.isLive() };
			def lovercount = [];
			def lovermax = 0;
			lover.each {
				if(lovercount[it.lovers] == null) lovercount[it.lovers] = 1;
				lovercount[it.lovers]++;
				if(lovercount[it.lovers] > lovermax) {
					lovermax = lovercount[it.lovers];
				}
			}
			if(lovermax >= 2) {
				return (result == 3) || (result == 4);	//恋人
			} else {
				return false;
			}
		}
		return (result == 2) || (result == 4);		//狐の場合
	}
}
