package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Runaway
// Camp: Civilians

class Runaway extends Civilian {
	def static order = 13;
	def static strings = Civilian.strings.attachBundle("roles/Runaway.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	Runaway(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 15;
	}
	def getRoleName() {
		return strings.getString("runaway");
		//runaway=逃亡者
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				notice(strings.getString("notice.runaway"));
				//notice.runaway=逃亡先を「逃亡＠対象の名前」の形式で夜時間の間に決定してください。
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.runaway") + "@" + it.name;
						//command.runaway=逃亡
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(type != strings.getString("command.runaway")) {
					return;
				}
				if(this.target) {
					//command.runaway=逃亡
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.runaway.target", target.name));
					//notice.runaway.target=逃亡先→%Sさん
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day == 0) {
					return true;
				}
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
	// Goes to next phase.
	def update() {
		super.update();
		if(JinroGame.day == 1) {
			return;
		}
		if(JinroGame.time == 1) {
			return;
		}
		if(Wolf.bitetarget == target || target.isWolf()) {	//逃亡先が噛まれた　又は　逃亡先が狼
			killed(State.followed2);
		}
	}
	
	// When the actor is killed.
	//状態を変化させ、それに伴う処理を行います。死亡時は死者のリストに追加し、処刑以外の死因であれば通知するようにします。
	//吊りの場合はgame.groovyのexecutionでメッセージを出しているのでここでは出さない
	//射殺やラブガードではupdateでメッセージを出してるのでここでは出さない
	def killed(state) {
		def actors = JinroGame.getActors();
		if(lovers == 0) {						//恋人以外
			if( (state != State.executed) && (state != State.shooting) && (state != State.loveguard) && (state != State.deathnote) ) {
				if(JinroGame.day >= 2 && state == State.bitten) {	//噛まれても死なない？（但し欠けはある）
					return;
				}
				if(this.state == State.live) {
					if( (deathnoteflg == false) || (state == State.followed) ) {//デスノで死んだら無残なを出さないが、道連れなら出す
						noticeKilled(strings.getString("notice.killed.normal", name));
						//notice.killed.normal=%Sさんが無残な死体となって発見されました・・・。
					}
				}
			}
		} else {								//恋人の時
			def lover = actors.findAll { (it.lovers == this.lovers) && (it.name != name) };
			def loversfellow = JinroGame.getProperty("loversfellow");
			if(state != State.executed) {
				if(JinroGame.day >= 2 && state == State.bitten) {	//噛まれても死なない？（但し欠けはある）
					return;
				}
			/*	if(state == State.followed2) {		//噛みの場合
					def wo = actors.findAll { it.isWolf() };
					wo.each {	//噛みを実行した狼を道連れ
						if( (it.state == State.live) && (loversfellow == true) ) {
							if(it.bitetarget2?.name == this.name) {
								it.killed(State.followed);		//道連れ
							} 
						}
					}
				}*/
				//吊り以外での恋人の後追い自殺
				switch(JinroGame.getProperty("ktime")) {
					case 0:	//投票後、猫の道連れで死んだ場合
						lover.each {
							if(it.state == State.live) {
								it.nextstate = State.followup;
								it.countdown = 2;
							}
						}
						break;
					case 1:	//明け方死んだ場合
						this.loverslive = false;						//無限ループ対策
						lover.each {
					//		JinroGame.addLog("aa name="+it.name+" state="+it.state);
							if((it.state == State.live) && (it.loverslive)) {
								it.killed(State.followup);				//後追い
							}
						}
						break;
				}
				if( (state != State.shooting) && (state != State.loveguard) && (state != State.deathnote) ) {
					if(this.state == State.live) {
						if( (deathnoteflg == false) || (state == State.followed) ) {//デスノで死んだら無残なを出さないが、道連れなら出す
							if(state == State.followup) {	//恋人後追いの時
								noticeKilled(strings.getString("notice.killed.lovers.followup", name) );
								//notice.killed.lovers.followup=%Sさんは恋人の後を追い自殺しました
							} else {
								noticeKilled(strings.getString("notice.killed.normal", name) );
								//notice.killed.normal=%Sさんが無残な死体となって発見されました・・・。
							}
						}
					}
				}
			} else {							//吊りの場合
				def lives = actors.findAll { it.isLive() && (it.name != name) && (it.voteforlovers == true) };
				if( lives && (loversfellow == true) )  {	//投票した人を道連れ
					def choice = lives[Math.floor(JinroGame.invokeMethod("rand") * lives.size()) as int];
					choice.nextstate = State.followed;
					choice.countdown = 2;
				}
				lover.each {	//恋人の後追い自殺
					if(it.state == State.live) {
						it.nextstate = State.followup;
						it.countdown = 2;
					}
				}
			}
		}
		this.killtime = JinroGame.getProperty("time0");
		this.state = state;
		JinroGame.addHistory(this);
	}
/*	def killed(state) {
		if(state != State.executed) {
			if(JinroGame.day >= 2 && state == State.bitten) {	//噛まれても死なない？（但し欠けはある）
				return;
			}
			noticeKilled(strings.getString("notice.killed.normal", name));
			//notice.killed.normal=%Sさんが無残な死体となって発見されました・・・。
		}
		this.state = state;
		JinroGame.addHistory(this);
	}*/
	
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	//勝利結果 0=村 1=狼 2=狐 3=恋人 4=狐＋恋人 return true=勝利 false=敗北
	def judge(result) {
		if(lovers != 0) {
			def lover = JinroGame.getActors().findAll { ((it.lovers & this.lovers) != 0) && it.isLive() };
			def lovercount = [];
			def lovermax = 0;
			lover.each {
				if(lovercount[it.lovers] == null) lovercount[it.lovers] = 1;
				lovercount[it.lovers]++;
				if(lovercount[it.lovers] > lovermax) {
					lovermax = lovercount[it.lovers];
				}
			}
			if(lovermax >= 2) {
				return (result == 3) || (result == 4);	//恋人
			} else {
				return false;
			}
		}
		return result == 0 && isLive();				//村＆生存
	}
}
