package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Sharer
// Camp: Civilians

class Sharer extends Civilian {
	def static order = 6;
	def static strings = Civilian.strings.attachBundle("roles/Sharer.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		if(number > 12) {
			return 2;
		} else {
			return 0;
		}
	}
	Sharer(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 35;
	}
	def getRoleName() {
		return strings.getString("sharer");
	}
	// Notifies the actor of the role.
	//参加者に対し役職の通知を行います。共有者は相方も通知します。
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.name != name && it.roleIs(this) }.each {
			notice(strings.getString("notice.partner", it.name));
		};
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				def deviltrigger = JinroGame.getProperty("deviltrigger");
				if(deviltrigger == true) {
					notice(strings.getString("notice.loveguard"));
					//notice.loveguard=ラブガード先を「ラブガード＠対象の名前」の形式で夜時間の間に決定してください。
				}
				break;
		}
	}
	// Returns the actor's command list
	//ツール本体から手動でコマンドを選択するために、そのリストを返します。
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				def deviltrigger = JinroGame.getProperty("deviltrigger");
				if(deviltrigger != true) {
					break;
				}
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.loveguard") + "@" + it.name;
						//command.loveguard=ラブガード
					};
				}
				break;
		}
		return commands;
	}
	//コマンドを処理します。昼時間の投票の他、夜時間のラブガードを処理します。
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				def deviltrigger = JinroGame.getProperty("deviltrigger");
				if(deviltrigger != true) {
					return;
				}
				if(type != strings.getString("command.loveguard")) {
					//command.loveguard=ラブガード
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.loveguard.target", target.name));
					//notice.loveguard.target=ラブガード先→%Sさん
					target.loveguard = true;
					
					//他の共有のターゲットにもセット
					def actors = JinroGame.getActors();
					actors.findAll { it.roleIs(new Sharer()) }.each{
						it.target = target;
					};
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	//ゲームを進められるかどうかを返します。必要な能力の使用が済んだかどうかなどで判断します。
	//時間を進めることが出来るならtrue、出来ない場合はfalse。
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day == 0) {
					return true;
				}
				def deviltrigger = JinroGame.getProperty("deviltrigger");
				if(deviltrigger != true) {
					return true;
				}
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
}
