package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Wolf (Werewolf)
// Camp: Wolves

class Wolf extends Civilian {
	def static order = 1;	//													役職の順番を表します。
	def static strings = Civilian.strings.attachBundle("roles/Wolf.properties");
	def static number = 0;	// All roles must define own "number" property.		この役職の人数を表します。
	def static bite = true;	// All roles must define own "bite" property.		この役職が初日犠牲者として死亡しないかどうかを表します。
	def static isInvisible() {
		return true;
	}
	// Returns the proper number.
	def static getJustnumber(number) {
		if(number > 19) {
			return 4;
		} else if(number > 15) {
			return 3;
		} else if(number > 8) {
			return 2;
		} else {
			return 1;
		}
	}
	def getPriority() {
		return 100;
	}
	Wolf(actor = null) {
		super(actor);
	}
	def static setBitetarget(target) {
		JinroGame.setProperty("bitetarget", target);
	};
	def static getBitetarget() {
		return JinroGame.getProperty("bitetarget");
	}
	def getRoleName() {
		return strings.getString("wolf");
		//wolf=狼
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				notice(strings.getString("notice.bite"));
				//notice.bite=襲撃先を「噛み＠対象の名前」の形式で夜時間の間に決定してください。
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(!bitetarget) {
					actors.findAll{ !it.isWolf() && it.isLive() }.each {
						commands << strings.getString("command.bite") + "@" + it.name;
						//command.bite=噛み
					};
				}
				break;
		}
		return commands;
	}
	// Checks if the game can go to next phase.
	//時間を進めることが出来るならtrue、出来ない場合はfalse。
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 0:
				return true;
				break;
			case 1:
				if(JinroGame.day == 0) { return true; }
				def wolves = JinroGame.getActors().findAll { it.isWolf() && it.bitetarget2 != null };
				if( wolves ) {
					return true;
				}
				break;
		}
		return false;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		// 0: daytime, 1: nighttime
		switch(JinroGame.time + JinroGame.getProperty("firstdaytime")*2) {
			case 1:
				if(type != strings.getString("command.bite")) {
				//command.bite=噛み
					return;
				}
				if(JinroGame.day == 0) {
					return;
				}
				if(bitetarget) {
					return;
				}
				if(!target?.isWolf() && target?.isLive()) {
					bitetarget = target;
					this.bitetarget2 = target;		//後で誰が噛んだかわかるようにセット
					notice(strings.getString("notice.bite.target", target.name));
					//notice.bite.target=噛み先→%Sさん
					if(JinroGame.getProperty("wolveschat") != null) {
						JinroGame.getProperty("wolveschat").send(strings.getString("notice.bite.target", target.name));
					}
				}
				break;
		}
	}
	// Notifies the actor of the role.
	//参加者に対し役職の通知を行います。
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.name != name && it.isWolf() }.each {
			notice(strings.getString("notice.wolf.partner", it.name, JinroGame.getAlias( it.getRoleName() )));
			//notice.wolf.partner=%1$Sさんが%2$Sです。
		}
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	//勝利結果 0=村 1=狼 2=狐 3=恋人 4=狐＋恋人 return true=勝利 false=敗北
	def judge(result) {
		if(lovers != 0) {
			def lover = JinroGame.getActors().findAll { ((it.lovers & this.lovers) != 0) && it.isLive() };
			def lovercount = [];
			def lovermax = 0;
			lover.each {
				if(lovercount[it.lovers] == null) lovercount[it.lovers] = 1;
				lovercount[it.lovers]++;
				if(lovercount[it.lovers] > lovermax) {
					lovermax = lovercount[it.lovers];
				}
			}
			if(lovermax >= 2) {
				return (result == 3) || (result == 4);	//恋人
			} else {
				return false;
			}
		}
		return result == 1;
	}
	// If it returns true, the role count as a wolf.
	def isWolf() {
		return true;
	}
	// If it returns true, the role count as a civilian.
	def isCivilian() {
		return false;
	}
}
